# Установка и управление сервером PostgreSQL

Что такое *кластер БД*?
Короче говоря, **кластер БД** - это объединение баз данных, управляемых СУБД(экземляром сервера **PostgreSQL**).
![img.png](static/img.png)
**PostgreSQL** есть ни что иное как СУБД.

## Исходные коды PostgreSQL
*Установка из исходных кодов PostgreSQL это:*
 - Стабильная версия сервера
 - Можно использовать нестандартные параметры
 - Можно собрать на специфичной архитектуре

*Установка PostgreSQL из репозитория Git это:*
- Текущая версия ядра (самая свежая)
- Необходима прежде всего для разработчиков ядра
- Требует более широкого набора инструментов

При сборке **сервера** из _Git_ генерятся файлы на **C**, а затем они компилируются *(это страшно)*
В отличие от **сервера** из исходных кодов - там уже скомпилированные C файлы! *(это круто)*

## Необходимые вещи для сборки исходного кода
![img.png](static/img.png)

## Создание конфигурации
 
В команде `configure` можно указать различные параметры конфигурации. Например: \
`--prefix` - каталог установки (по умолчанию /usr/local/pgsql)\
`--enable-debug` - для включения отладочной информации\
`--with-pgport` - pадает порт PostgreSQL по умолчанию\
`--with-python` - включает поддержку PL/Python на сервере\

Больше параметров можно посмотреть [тут](https://postgrespro.ru/docs/postgresql/9.5/install-procedure).

После создания конфигурации нужно выполнить команду `make` для сборки **сервера**
Есть еще `make world` - сборка сервера, всех расширений и документации

`make install` - копирование исполняемых файлов и библиотек в каталог указанный в `--prefix`.

Для сборки расширений можно повторить эти же две команды в подкаталоге `contrib`

## Утилита initdb

Новый кластер БД не может принадлежать суперпользователю.

`PGDATA` - удобная переменная для каталога кластера.

Файлы конфигурации создаются в каталоге PGDATA.\

_Желательно включить расчёт контрольных сумм для страниц с данными._

Нужно создать каталог для данных. 

```shell
mkdir /home/student/pgsql/data
```

Потом нужно добавить переменную окружения `PGDATA` с этим каталогом.

```shell
export PGDATA=/home/student/pgsql/data
```

Все утилиты сервера установлены в bin, поэтому добавим его в `PATH`

```shell
export PATH=/home/student/pgsql/bin:$PATH
```

Инициализируем кластер баз данных

```shell
initdb -U postgres -k -D /home/student/pgsql/data
```

В ключе `-U` можно указать суперпользователя СУБД\
Ключ `-k` включает подсчет контрольных сумм страницы. Это позволяет обнаруживать повреждение данных.\
Если переменная `PGDATA` не установлена, то нужно указывать каталог для данных.

## Утилита pg_ctl

Основные ее задачи - это запуск, остановка, получение статуса сервера и обновление параметров конфигурации.

### Запуск
```shell
pg_ctl start -l /home/student/logfile
```

Флаг `-l` указывает путь логов.

### Остановка
```shell
pg_ctl stop -m fast
```

В ключе `-m` можно указать один из трех режимов остановки:
- `fast` - принудительно завершает все сеансы и пишет на диск изменения из ОЗУ.
- `smart` - ждет, пока все сеансы будут завершены и пишет на диск изменения из ОЗУ. _(не дает создавать новые подключения)_
- `immediate` - принудительно завершает все сеансы, при запуске потребуется восстановление. _(Ничего не пишет из ОЗУ)_

## Установка из готовых пакетов (для Ubuntu)
Используется оболочка над `initdb` - `pg_createcluster`
Инициализация выполняется при установке пакета и создается кластер `main`

**Подсчет контрольных сумм по умолчанию выключен**

```shell
sudo apt install -y postgresql-13 
```

Утилита pg_config показывает, с какими параметрами был собран сервер
```shell
sudo /usr/lib/postgresql/13/bin/pg_config --configure
```

Кластер уже был инициализирован при установке postgres - `main`

Утилита `pg_ctlcluster` - обертка над `pg_ctl`.\
Ее назначение такое же.

Остановка сервера:
```shell
sudo pg_ctlcluster 13 main stop
```
Выполняется либо от суперпользователя, либо от пользователя postgres
```shell
sudo pg_ctlcluster 13 main start
```

```shell
sudo pg_ctlcluster 13 main restart
```

```shell
sudo pg_ctlcluster 13 main status
```

`13` - номер версии сервера\
`main` - название сервера


Логи находятся тут:
```shell
ls -l /var/log/postgresql/postgresql-13-main.log
```

## Практическое задание
1. Остановить сервер
2. Проверить, считаются ли контрольные суммы
3. Включить расчет контрольных сумм
4. Запуск сервера

### Остановка сервера
```shell
sudo pg_ctlcluster 13 main stop -m fast
```

### Проверка подсчета контрольных сумм
```shell
sudo /usr/lib/postgresql/13/bin/pg_checksums 
-D /var/lib/postgresql/13/main/
```

`error: data checksums are not enabled in cluster`\
Они выключены.

### Включение расчета контрольных сумм
```shell
sudo /usr/lib/postgresql/13/bin/pg_checksums 
 -D /var/lib/postgresql/13/main/
--enable
```

### Включение сервера
```shell
sudo pg_ctlcluster 13 main start
```

 