### Подключение
```shell
psql -d база -U пользователь -h узел -p порт
```

Посмотреть информацию о подключении можно командой

```shell
\conninfo
```

В конце команды должна быть `;`

Некоторые команды для переключения режима форматирования:
- `\t` - режим tuples only (выводить только кортежи данных)
- `\a` - выключить выравнивание
- `\pset fieldsep is " " ` - смена сепаратора
- `\x` - расширенный формат
- `\pset` - показывает текущие настройки

Можно записывать результаты выполнения команд в файл 

`\o имя_файла`

Можно устанавливать переменные 
```shell
\set TEST Hello!
```

```shell
\echo :TEST
```

Hello!

И удалять их

```shell
\unset TEST
```
Посмотреть все переменные можно командой
```shell
\set
```

Проверить, определена ли переменная можно конструкцией 
```shell
\echo :{?var_name}
```

## Условный оператор

```shell
\if :{?working_dir}
  \echo "ВСЕ ОК"
\else
  \echo "ВСЕ ПЛОХО"
  \set working_dir `pwd`
\endif
```

### Практика
- Запустите psql и проверьте информацию о подключении
```shell
psql -U postgres -h localhost -p 5432
```

```shell
\conninfo
```

- Выведите все строки таблицы pg_tables.

```shell
SELECT * FROM pg_tables;
```

- Установите команду less -XS для постраничного просмотра и еще раз выведите все строки pg_tables
```shell
\setenv PSQL_PAGER "less"
```

- Измените приглашение на роль@база=#
```shell
\set PROMPT1 '%n@%/=%#'
```

- `%n` - отображает имя пользователя
- `@` - разделяет имя пользователя и имя базы данных
- `%/` - отображает текущее имя базы данных
- `%#` - отображает # для суперпользователя или > для обычного пользователя



- Откройте транзакцию и выполните команду, которая завершается любой ошибкой.
```sql
BEGIN;
CREATE TABLE TEST(id SERIAL PRIMARY KEY not null, num INTEGER CHECK(num > 0));
INSERT INTO TEST(num) VALUES (-1);
COMMIT;
```

- Установите переменную ON_ERROR_ROLLBACK значение on

```shell
\set ON_ERROR_ROLLBACK 'on'
```
